@@ -21,6 +21,10 @@
 #ifndef _GIAC_FIRST_H_
 #define _GIAC_FIRST_H_
 
+#ifdef NUMWORKS
+#define KHICAS 1
+#endif
+
 #ifndef GIAC_VERSION
 #define GIAC_VERSION VERSION
 #endif
@@ -55,6 +59,8 @@
 #define NO_STDEXCEPT 1
 #endif
 
+#define MAX_INTSTACK 32768 // maximal size for allocating an array by int tab[]
+
 #ifdef FXCG
 #define RAND_MAX 2147483647
 #define clock() 0
@@ -128,6 +134,8 @@ typedef long double giac_double;
 typedef double giac_double;
 #endif
 
+typedef long double  long_double;
+
 // sprintf replacement
 int my_sprintf(char * s, const char * format, ...);
 #ifdef GIAC_HAS_STO_38
@@ -137,7 +145,7 @@ int my_sprintf(char * s, const char * format, ...);
 #ifdef WITH_MYOSTREAM
 #include "myostream.h"
 #else
-#if defined NUMWORKS //&& defined STATIC_BUILTIN_LEXER_FUNCTION
+#if defined KHICAS //&& defined STATIC_BUILTIN_LEXER_FUNCTION
 #include "stdstream"
 #define my_ostream stdostream
 #else
@@ -162,7 +170,7 @@ int my_sprintf(char * s, const char * format, ...);
 #define CLOCK_T int
 #endif
 
-#if !defined HAVE_ALLOCA_H && !defined GIAC_HAS_STO_38 && !defined NUMWORKS
+#if !defined HAVE_ALLOCA_H && !defined GIAC_HAS_STO_38 && !defined KHICAS
 #define alloca _alloca
 #endif
 
@@ -277,8 +285,8 @@ typedef unsigned __int64 ulonglong ;
 typedef long long longlong;
 typedef unsigned long long ulonglong;
 #ifdef x86_64
-  typedef int int128_t __attribute__((mode(TI)));
-  typedef unsigned int uint128_t __attribute__((mode(TI)));
+typedef int int128_t __attribute__((mode(TI)));
+typedef unsigned int uint128_t __attribute__((mode(TI)));
 #ifndef INT128
 #define INT128 1
 #endif
@@ -298,6 +306,7 @@ typedef unsigned long long ulonglong;
 
 #endif // __VISUALC__
 
+
 #ifdef VISUALC
 inline void swap_giac_double(double & a,double & b){ double c=a; a=b; b=c; }
 #else
