@@ -29,7 +29,7 @@ namespace giac {
   extern bool user_screen; 
   extern int user_screen_io_x,user_screen_io_y,user_screen_fontsize;
   extern const int rand_max2; // replace RAND_MAX if giac_rand(contextptr) is used
-  extern bool warn_equal_in_prog;
+  extern bool warn_equal_in_prog,warn_symb_program_sto;
 
   struct user_function;
   struct module_info {
@@ -43,6 +43,7 @@ namespace giac {
   extern modules_tab giac_modules_tab;
 #endif
 
+  void alert(const std::string & s,GIAC_CONTEXT);
   gen check_secure(); // in secure mode error
   void set_decimal_digits(int n,GIAC_CONTEXT);
   int digits2bits(int n);
@@ -54,7 +55,7 @@ namespace giac {
   gen equaltosame(const gen & a);
   gen sametoequal(const gen & a);    
   gen equaltosto(const gen & g,GIAC_CONTEXT);
-  int bind(const vecteur & vals,const vecteur & vars,context * & contextptr);
+  int giac_bind(const vecteur & vals,const vecteur & vars,context * & contextptr);
   bool leave(int protect,vecteur & vars,context * & contextptr);
 
   void increment_instruction(const vecteur & v,debug_struct * dbgptr);
@@ -75,6 +76,8 @@ namespace giac {
   gen quote_program(const gen & args,GIAC_CONTEXT);
   gen _program(const gen & args,const gen & name,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_program ;
+  // parser helper
+  gen symb_test_equal(const gen & a,const gen & op,const gen & b);
   void adjust_sst_at(const gen & name,GIAC_CONTEXT); //used in symbolic.cc by nr_eval
   void program_leave(const gen & save_debug_info,bool save_sst_mode,debug_struct * dbgptr);
 
@@ -297,6 +300,9 @@ namespace giac {
   gen _all_trig_solutions(const gen & args,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_all_trig_solutions;
 
+  gen _increasing_power(const gen & args,GIAC_CONTEXT);
+  extern const unary_function_ptr * const  at_increasing_power;
+
   gen _ntl_on(const gen & args,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_ntl_on;
 
@@ -353,6 +359,7 @@ namespace giac {
 
   gen _sort(const gen & args,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_sort;
+  extern const unary_function_ptr * const  at_sorted;
 
   gen _ans(const gen & args,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_ans;
@@ -384,9 +391,13 @@ namespace giac {
   gen quote_read(const gen & args,GIAC_CONTEXT); // read in a file and return non evaled
   gen _read(const gen & args,GIAC_CONTEXT); // read in a file and return evaled
   extern const unary_function_ptr * const  at_read;
+  extern const unary_function_ptr * const  at_read16;
+  extern const unary_function_ptr * const  at_read32;
 
   gen _write(const gen & args,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_write;
+  extern const unary_function_ptr * const  at_write16;
+  extern const unary_function_ptr * const  at_write32;
 
   gen _save_history(const gen & args,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_save_history;
@@ -404,6 +415,7 @@ namespace giac {
   extern const unary_function_ptr * const  at_tableseq;
 
   gen protecteval(const gen & g,int level,GIAC_CONTEXT);
+  gen protectevalf(const gen & g,int level,GIAC_CONTEXT);
 
   gen _nodisp(const gen & args);
   extern const unary_function_ptr * const  at_nodisp;
@@ -440,7 +452,7 @@ namespace giac {
   gen _inputform(const gen & args,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_inputform;
   class unary_function_eval;
-#if !defined RTOS_THREADX && !defined NSPIRE
+#if !defined RTOS_THREADX && !defined NSPIRE && !defined FXCG
   extern unary_function_eval __inputform;
 #endif
 
@@ -488,13 +500,13 @@ namespace giac {
 
   gen _Row(const gen & args,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_Row;
-#if !defined RTOS_THREADX && !defined NSPIRE
+#if !defined RTOS_THREADX && !defined NSPIRE && !defined FXCG
   extern unary_function_eval __Row;
 #endif
 
   gen _Col(const gen & args,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_Col;
-#if !defined RTOS_THREADX && !defined NSPIRE
+#if !defined RTOS_THREADX && !defined NSPIRE && !defined FXCG
   extern unary_function_eval __Col;
 #endif
 
@@ -521,7 +533,7 @@ namespace giac {
   // used to update IO screen and graph inside progs
   gen _interactive(const gen & args,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_interactive;
-#if defined RTOS_THREADX || defined NSPIRE
+#if defined RTOS_THREADX || defined NSPIRE || defined FXCG
   extern const alias_unary_function_eval __interactive;
   // extern const unary_function_eval __interactive;
 #else
@@ -546,7 +558,7 @@ namespace giac {
   gen _ti_semi(const gen & args,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_ti_semi;
 
-#if defined RTOS_THREADX || defined NSPIRE
+#if defined RTOS_THREADX || defined NSPIRE || defined FXCG
   // extern const unary_function_eval __keyboard;
 #else
   extern unary_function_eval __keyboard;
@@ -554,20 +566,20 @@ namespace giac {
   extern const unary_function_ptr * const  at_keyboard;
   gen widget_size(const gen & g,GIAC_CONTEXT);
   gen keyboard(const gen & g,GIAC_CONTEXT);
-#if !defined RTOS_THREADX && !defined NSPIRE
+#if !defined RTOS_THREADX && !defined NSPIRE && !defined FXCG
   extern unary_function_eval __widget_size;
 #endif
   extern const unary_function_ptr * const  at_widget_size;
 
   gen current_sheet(const gen & g,GIAC_CONTEXT);
-#if !defined RTOS_THREADX && !defined NSPIRE
+#if !defined RTOS_THREADX && !defined NSPIRE && !defined FXCG && !defined KHICAS
   extern unary_function_eval __current_sheet;
 #endif
   extern const unary_function_ptr * const  at_current_sheet;
 
   gen window_switch(const gen & g,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_window_switch;
-#if !defined RTOS_THREADX && !defined NSPIRE
+#if !defined RTOS_THREADX && !defined NSPIRE && !defined FXCG
   extern const unary_function_eval __window_switch;
   extern const unary_function_eval __maple_lib;
 #endif
@@ -618,6 +630,7 @@ namespace giac {
   vecteur mksa_convert(const gen & g,GIAC_CONTEXT);
   gen _ufactor(const gen & g,GIAC_CONTEXT);
   gen _usimplify(const gen & g,GIAC_CONTEXT);
+  extern const unary_function_ptr * const  at_regrouper;  
 
   extern const mksa_unit __m_unit;
   extern const mksa_unit __kg_unit;
@@ -803,6 +816,7 @@ namespace giac {
   extern gen _mol_unit;
   extern gen _cd_unit;
   extern gen _E_unit;
+#ifndef STATIC_BUILTIN_LEXER_FUNCTIONS
   // other metric units in m,kg,s,A
   extern gen _Bq_unit;
   extern gen _C_unit;
@@ -952,9 +966,12 @@ namespace giac {
   extern gen cst_Vm;
   extern gen cst_kBoltzmann;
   extern gen cst_NA;
+#endif // STATIC_BUILTIN_LEXER_FUNCTIONS
 #endif // NO_PHYSICAL_CONSTANTS
+#ifndef FXCG
   const unary_function_ptr * binary_op_tab();
-
+#endif
+  
   extern const unary_function_ptr * const  at_piecewise;
   extern const unary_function_ptr * const  at_PIECEWISE;
   gen _piecewise(const gen & g,GIAC_CONTEXT);
@@ -999,9 +1016,26 @@ namespace giac {
 
   gen _autosimplify(const gen & g,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_struct_dot ;
+  gen _struct_dot(const gen & g,GIAC_CONTEXT);
+  // replace := by = in builtin commands (for Python compatible mode)
+  gen denest_sto(const gen & g);
+  gen os_nary_workaround(const gen & g); // replace [[a,b,...]] by a,b,...
 
   extern const unary_function_ptr * const  at_index ;
   gen _index(const gen & args,GIAC_CONTEXT);
+  extern const unary_function_ptr * const  at_extend ;
+  extern const unary_function_ptr * const  at_python_compat ;
+  extern const unary_function_ptr * const  at_randint ;
+  extern const unary_function_ptr * const  at_choice ;
+  extern const unary_function_ptr * const  at_randrange ;
+  extern const unary_function_ptr * const  at_giac_assert ;
+  extern const unary_function_ptr * const  at_heapify ;
+  extern const unary_function_ptr * const  at_heappop ;
+  extern const unary_function_ptr * const  at_heappush ;
+  extern const unary_function_ptr * const  at_giac_bool ;
+  extern const unary_function_ptr * const  at_shuffle ;
+  extern const unary_function_ptr * const  at_giac_bin ;
+  extern const unary_function_ptr * const  at_giac_hex ;
 
 #ifndef NO_NAMESPACE_GIAC
 } // namespace giac
