@@ -308,6 +308,8 @@ namespace giac {
   extern const unary_function_ptr * const  at_normalmod;
 
   symbolic symb_sto(const gen & a,gen & b,bool in_place=false);
+  symbolic parser_symb_sto(const gen & a_,gen & b,bool in_place=false);
+  // same as symb_sto but check if a is a vector, if so make a copy
   symbolic symb_sto(const gen & e);
   extern const unary_function_ptr * const  at_sto ;
   extern const unary_function_ptr * const  at_array_sto ;
@@ -321,6 +323,7 @@ namespace giac {
   extern const unary_function_ptr * const  at_orsto;
   extern const unary_function_ptr * const  at_xorsto;
   extern const unary_function_ptr * const  at_shiftsto;
+  extern const unary_function_ptr * const  at_powsto;
   gen _shiftsto(const gen & a,const context * contextptr);
   extern const unary_function_ptr * const  at_rotatesto;
   gen _rotatesto(const gen & a,const context * contextptr);
@@ -498,6 +501,9 @@ namespace giac {
   extern const unary_function_ptr * const  at_Si;
   extern const unary_function_ptr * const  at_Ei ;
   extern const unary_function_ptr * const  at_Ei_f ;
+  extern const unary_function_ptr * const  at_LambertW;
+  extern const unary_function_ptr * const  at_LambertWs;
+  gen _LambertW(const gen & args,GIAC_CONTEXT);
 
   gen Psi(const gen & x,GIAC_CONTEXT);
   gen Psi(const gen & x,int n,GIAC_CONTEXT);
