@@ -45,6 +45,8 @@ namespace giac {
   typedef double long_double;
 #endif
   typedef std::complex<long_double> complex_long_double;
+  double complex_abs(const complex_double & c);
+  double complex_long_abs(const complex_long_double & c);
 
   // make a matrix with free rows 
   // (i.e. it is possible to modify the answer in place)
@@ -157,7 +159,7 @@ namespace giac {
   // extract submatrix
   matrice matrice_extract(const matrice & m,int insert_row,int insert_col,int nrows,int ncols);
   void makespreadsheetmatrice(matrice & m,GIAC_CONTEXT);
-  matrice extractmatricefromsheet(const matrice & m);
+  matrice extractmatricefromsheet(const matrice & m,bool value=true);
   // eval spreadsheet, compute list of dependances in lc
   void spread_eval(matrice & m,GIAC_CONTEXT);
 
@@ -197,7 +199,7 @@ namespace giac {
   // if a and b are matrices returns matrix product
   // if a is a matrix and b a vecteur return matr*vect
   // otherwise returns the dot product of a and b
-  gen ckmultmatvecteur(const vecteur & a,const vecteur & b);
+  gen ckmultmatvecteur(const vecteur & a,const vecteur & b,GIAC_CONTEXT);
   void multmatvecteur(const matrix_double & H,const std::vector<giac_double> & w,std::vector<giac_double> & v);
 
   void vecteur2vector_int(const vecteur & v,int modulo,std::vector<int> & res);
@@ -229,6 +231,7 @@ namespace giac {
 
   // mmult assumes dimensions are correct
   void mmult(const matrice & a,const matrice & b,matrice &res);
+  void mmult_atranb(const matrice & a,const matrice & btran,matrice & res);
   matrice mmult(const matrice & a,const matrice & b);
   bool mmultck(const matrice & a,const matrice & b,matrice & res);
   matrice mmultck(const matrice & a,const matrice & b);
@@ -268,7 +271,8 @@ namespace giac {
     RREF_LAGRANGE=5
   };
   // For approx linear combination, anything ||<eps will be replaced by 0
-  void linear_combination(const gen & c1,const vecteur & v1,const gen & c2,const vecteur & v2,const gen & c,vecteur & v,double eps,int cstart=0);
+  void linear_combination(const gen & c1,const vecteur & v1,const gen & c2,const vecteur & v2,const gen & c,const gen & cinv,vecteur & v,double eps,int cstart=0);
+  gen exact_div(const gen & a,const gen & b);
 
   // row reduction from line l and column c to line lmax and column cmax
   // lmax and cmax are not included
@@ -293,23 +297,30 @@ namespace giac {
     std::vector<int> y,y1,y2,y3;
     std::vector<longlong> z,z1,z2,z3;
   };
-  bool in_modrref(const matrice & a, std::vector< std::vector<int> > & N,matrice & res, vecteur & pivots, gen & det,int l, int lmax, int c,int cmax,int fullreduction,int dont_swap_below,int modulo,int rref_or_det_or_lu,const gen & mult_by_det_mod_p,bool inverting,bool no_initial_mod,smallmodrref_temp_t * workptr=0);
+  bool in_modrref(const matrice & a, std::vector< std::vector<int> > & N,matrice & res, vecteur & pivots, gen & det,int l, int lmax, int c,int cmax,int fullreduction,int dont_swap_below,int modulo,int carac,int rref_or_det_or_lu,const gen & mult_by_det_mod_p,bool inverting,bool no_initial_mod,smallmodrref_temp_t * workptr);
   bool modrref(const matrice & a, matrice & res, vecteur & pivots, gen & det,int l, int lmax, int c,int cmax,
-	       int fullreduction,int dont_swap_below,const gen & modulo,int rref_or_det_or_lu);
+	       int fullreduction,int dont_swap_below,const gen & modulo,bool ckprime,int rref_or_det_or_lu);
   bool mrref(const matrice & a, matrice & res, vecteur & pivots, gen & det,GIAC_CONTEXT);
   bool modrref(const matrice & a, matrice & res, vecteur & pivots, gen & det,const gen& modulo);
-  bool modrref(const matrice & a, matrice & res, vecteur & pivots, gen & det,int l, int lmax, int c,int cmax,int fullreduction,int dont_swap_below,const gen & modulo,int rref_or_det_or_lu);
+  
 
   // finish full row reduction to echelon form if N is upper triangular
   // this is done from lmax-1 to l
   void smallmodrref_upper(std::vector< std::vector<int> > & N,int l,int lmax,int c,int cmax,int modulo);
+  // finish row reduction for matrices with much more columns than rows
+  // version adapted for threads parallelization
+  // assumes that all columns are reduced in parallel, pivots are searched
+  // starting at column 0
+  void in_thread_smallmodrref_upper(std::vector< std::vector<int> > & N,int l,int lpivot,int lmax,int c,int cmax,int modulo,int parallel);
+  void thread_smallmodrref_upper(std::vector< std::vector<int> > & N,int l,int lmax,int c,int cmax,int modulo,int parallel);
   void free_null_lines(std::vector< std::vector<int> > & N,int l,int lmax,int c,int cmax);
+  int smallmodrref_lastpivotcol(const std::vector< std::vector<int> > & K,int lmax);
 
-  void smallmodrref(int nthreads,std::vector< std::vector<int> > & N,vecteur & pivots,std::vector<int> & permutation,std::vector<int> & maxrankcols,longlong & idet,int l, int lmax, int c,int cmax,int fullreduction,int dont_swap_below,int modulo,int rref_or_det_or_lu,bool reset=true,smallmodrref_temp_t * workptr=0,bool allow_block=true);
+  void smallmodrref(int nthreads,std::vector< std::vector<int> > & N,vecteur & pivots,std::vector<int> & permutation,std::vector<int> & maxrankcols,longlong & idet,int l, int lmax, int c,int cmax,int fullreduction,int dont_swap_below,int modulo,int rref_or_det_or_lu,bool reset,smallmodrref_temp_t * workptr,bool allow_block,int carac);
   void doublerref(matrix_double & N,vecteur & pivots,std::vector<int> & permutation,std::vector<int> & maxrankcols,double & idet,int l, int lmax, int c,int cmax,int fullreduction,int dont_swap_below,int rref_or_det_or_lu,double eps);
   void modlinear_combination(vecteur & v1,const gen & c2,const vecteur & v2,const gen & modulo,int cstart,int cend=0);
   void modlinear_combination(std::vector<int> & v1,int c2,const std::vector<int> & v2,int modulo,int cstart,int cend,bool pseudo);
-  vecteur fracmod(const vecteur & v,const gen & modulo);
+  vecteur fracmod(const vecteur & v,const gen & modulo,gen * den=0,int prealloc=128);
   gen modproduct(const vecteur & v, const gen & modulo);
   matrice mrref(const matrice & a,GIAC_CONTEXT);
   gen _rref(const gen & a,GIAC_CONTEXT); // first non 0 elem in row is 1
@@ -319,6 +330,7 @@ namespace giac {
   bool remove_identity(matrice & res);
   bool remove_identity(std::vector< std::vector<int> > & res,int modulo);
 
+  void mdividebypivot(matrice & a,int lastcol,GIAC_CONTEXT); // in-place div by pivots
   void mdividebypivot(matrice & a,int lastcol=-1); // in-place div by pivots
   // if lastcol==-1, divide last col, if lastcol==-2 do not divide last col
   // if lastcol>=0 stop dividing at lastcol
@@ -327,6 +339,7 @@ namespace giac {
   gen _idn(const gen & e,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_idn ;
 
+  gen fieldcoeff(const gen &F);
   vecteur vranm(int n,const gen & f,GIAC_CONTEXT); 
   matrice mranm(int n,int m,const gen & f,GIAC_CONTEXT); // random matrix using f
   gen _ranm(const gen & e,GIAC_CONTEXT);
@@ -371,9 +384,13 @@ namespace giac {
   gen _hessenberg(const gen & g,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_hessenberg ;
 
-  bool balance_krylov(matrix_double & H,std::vector<giac_double> & d,int niter=5,double cutoff=1e-8);
+  bool balance_krylov(const matrix_double & H,std::vector<giac_double> & d,int niter=5,double cutoff=1e-8);
   bool probabilistic_pmin(const matrice & m,vecteur & w,bool check,GIAC_CONTEXT);
-  bool mod_pcar(std::vector< std::vector<int> > & N,int modulo,bool & krylov,std::vector<int> & res,GIAC_CONTEXT);
+  vecteur mpcar_int(const matrice & A,bool krylov,GIAC_CONTEXT,bool compute_pmin);
+
+  void mod_pcar(std_matrix<gen> & N,vecteur & res,bool compute_pmin);
+  bool mod_pcar(const matrice & A,std::vector< std::vector<int> > & N,int modulo,bool & krylov,std::vector<int> & res,GIAC_CONTEXT,bool compute_pmin);
+  bool mod_pcar(std::vector< std::vector<int> > & N,int modulo,bool & krylov,std::vector<int> & res,GIAC_CONTEXT,bool compute_pmin);
   vecteur mpcar_hessenberg(const matrice & A,int modulo,GIAC_CONTEXT);
   gen _pcar_hessenberg(const gen & g,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_pcar_hessenberg ;
@@ -460,6 +477,9 @@ namespace giac {
   extern const unary_function_ptr * const  at_qr ;
   matrice thrownulllines(const matrice & res);
 
+  extern const unary_function_ptr * const  at_lll_reduce ;
+  extern const unary_function_ptr * const  at_lll ;
+
   gen _cholesky(const gen & a,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_cholesky ;
   gen _svd(const gen & a,GIAC_CONTEXT);
@@ -485,6 +505,7 @@ namespace giac {
 
   void matrice2std_matrix_gen(const matrice & m,std_matrix<gen> & M);
   void std_matrix_gen2matrice(const std_matrix<gen> & M,matrice & m);
+  void std_matrix_gen2matrice_destroy(std_matrix<gen> & M,matrice & m);
 
   bool is_integer_vecteur(const vecteur & m,bool intonly=false);
   bool is_integer_matrice(const matrice & m,bool intonly=false);
@@ -500,7 +521,7 @@ namespace giac {
   bool hermite(const std_matrix<gen> & Aorig,std_matrix<gen> & U,std_matrix<gen> & A,environment * env,GIAC_CONTEXT);
   gen _ihermite(const gen & g,GIAC_CONTEXT);
   gen _ismith(const gen & g,GIAC_CONTEXT);
-#ifndef NSPIRE
+#if !defined NSPIRE && !defined FXCG
   gen _csv2gen(const gen & g,GIAC_CONTEXT);
   matrice csv2gen(std::istream & i,char sep,char nl,char decsep,char eof,GIAC_CONTEXT);
 #endif
